#
# Usage: cceGenerateDoc [--html] [--man]
#

set documentation {
    [title cceExtHtml]

    [heading NAME]
    [paragraph {cceExtHtml - Extract HTML documentation from C++ source
and header files}]

    [heading SYNOPSIS]

    [tt cceExtHtml]
    [optional [optionval -t title]]
    [optional [optionval --title title]]
    [optional [optionval --prefix dir]]
    [optional [option -f]]
    [optional [option --fields]]
    [optional [option -v]]
    [optional [option --version]]
    [optional [option -h]]
    [optional [option --help]]
    [it files]

    [heading DESCRIPTION]

    [paragraph "[tt cceExtHtml] extracts HTML documentation from C++
source and header files.  It understands a subset of C++
(the main difference is that types of members and method
arguments must be ``simple''), and looks for
specially-formatted comments in certain places."]

    [heading OPTIONS]

    [defs "
       [defterm [optionval -t title]]
       [defterm [optionval --title title]]
       [defdef "Specify the title of the system being documented."]

       [defterm [optionval --prefix dir]]
       [defdef "Write files to the given directory.  Defaults to the current working directory."]

       [defterm [option -f]]
       [defterm [option --fields]]
       [defdef "Print the list of recognized comment fields."]

       [defterm [option -v]]
       [defterm [option --version]]
       [defdef "Print version information."]

       [defterm [option -h]]
       [defterm [option --help]]
       [defdef "Print command usage information."]
     "]

    [heading EXAMPLES]

    [paragraph "Extract documentation from all source and header files
in [tt /users/ptolemy/src/kernel]"]
    [blockquote [tt "% cceExtHtml --title \"The Ptolemy Kernel\" /users/ptolemy/src/kernel/*.{cc,h}"]]

    [heading "CODING STYLE"]

    [paragraph "[tt cceExtHtml] parses a subset of C++, recognizing class
and structure declarations and method definitions.  The contents of
function bodies, enumeration, typedef and extern declarations are
ignored.  Comments are first-class entities, limiting where they
may appear in class declarations outside function bodies.
In particular, a comment immediately preceding a [tt public:],
[tt protected:], or [tt private:] declaration within a class definition is
considered a syntax error."]

    [paragraph "In general, place all comments [emph before] the
entity they comment.  In particular, [tt cceExtHtml] will mistakenly
associate a comment after a variable declaration on the same line with
the next entity.  Comments within function arguments are the one
exception to this rule, which much appear [emph after] the agument,
but before the comma that separates it from the next argument."]

    [paragraph "When both a declaration and a definition of an entity,
such as a method, appear, the comment for the definition takes
precedence."]

    [paragraph "Comments may be of two forms: old-style comments,
which are surrounded by /* */, and new-style comments, which are one
or more lines that begin with //.  By default, the text within these
comments is interpreted as a short synopsis of the entity it precedes.
Additional fields within a comment are denoted by a keyword starting
with an at-sign.  The text associated with a field starts immediately
after the keyword and continues until the next such keyword or until
the end of the comment.  Invoke [tt cceExtHtml] with the [option --fields]
option to see the list of recognized fields."]

    [paragraph "Some sample comments are shown below"]

    [pre {
// A short synopsis in a new style comment

// A one-sentence description
//
// @Description With a longer, potentially multi-line
//              description below it in a new-style
//              comment block

/* Normal old-style comment */

/******************************

  Example of ignoring asterisks

  @Description Asterisks immediately after the start or immediately
               before the end of an old-style comment are ignored.

  @Author Stephen Edwards

 ******************************/
}]

    [paragraph "[tt cceExtHtml] only parses a subset of C++.  In
particular, the types it recognizes must be ``simple.''  A class
variable, function argument, or return type must be either a simple
scalar, a class, or a type from a typedef.  Additionally, the type may
be a pointer (of any depth, e.g., a pointer to a pointer) to, a
reference to, or a one-dimensional array of any of these simple types.
More complex types (e.g., function types, multi-dimensional arrays)
should be typedefed."]

    [paragraph "Each variable declaration within a class definition
may only declare a single variable, e.g., [tt "int a,b;"] is not
permitted, but [tt "int a; int b;"] is."]

    [heading AUTHOR]

    [href "Stephen Edwards" "http://www.eecs.berkeley.edu/~sedwards"]
    [break]
    University of California, Berkeley
    [break]
    [mailaddress "sedwards@eecs.berkeley.edu"]

}

proc generate_html_documentation {doc} {
    global datestring
    proc title {t}	{ return "<Head><Title>$t</Title></Head>\n<Body>\n" }
    proc heading {h}	{ return "<H2>$h</H2>\n" }
    proc paragraph {p}	{ return "$p <P>\n" }
    proc tt {t}		{ return "<TT>$t</TT>" }
    proc it {i}		{ return "<I>$i</I>" }
    proc emph {e}	{ return "<EM>$e</EM>" }
    proc pre {p}	{ return "<PRE>\n$p\n</PRE>" }
    proc option {o}	{ return "<TT>$o</TT>" }
    proc optionval {o v} { return "<TT>$o</TT> <I>$v</I>" }
    proc optional {o}	{ return "\[ $o \] " }
    proc defs {d}	{ return "<DL>\n$d\n</DL>\n" }
    proc defterm {t}	{ return " <DT>$t\n" }
    proc defdef {d}	{ return " <DD>$d <P>\n" }
    proc blockquote {b}	{ return "<Blockquote>\n$b\n</Blockquote>\n" }
    proc href {t h}	{ return "<A HREF=\"$h\">$t</A>" }
    proc mailaddress {a} { return "<A HREF=\"mailto:$a\"><TT>$a</TT></A>" }
    proc break {}	{ return "<BR>\n" }

    return "<Html>\n[expr \"$doc\"]\n<HR>\nGenerated on ${datestring} by <A HREF=\"http://www.eecs.berkeley.edu/~sedwards/cce/\">cce</A>\n</Html>"
}

proc generate_troff_documentation {doc} {
    proc title {t}	{ global datestring
	return "\n.TH $t 1 \"${datestring}\"\n" }
    proc heading {h}	{ return "\n.SH $h\n" }
    proc paragraph {p}	{ return "\n.PP\n$p" }
    proc tt {t}		{ return "\n.B $t\n" }
    proc it {i}		{ return "\n.I $i\n" }
    proc emph {e}	{ return "\n.I $e\n" }
    proc pre {p}	{
	regsub -all "\n" $p " \"\n.br\n.B \"" p1
	return "\n.RS\n.sp\n.nf\n.B \"${p1}\"\n.fi\n.RE\n"
    }
    proc optional {o}	{ return "\[$o\]" }
    proc option {o}	{ return "\n.B $o\n" }
    proc optionval {o v} { return "\n.BI \"$o \" \"$v\"\n" }
    proc defs {d}	{ return "$d" }
    proc defterm {t}	{ return "\n.TP\n$t\n.br\n.ns\n" }
    proc defdef {d}	{ return "$d\n" }
    proc blockquote {b}	{ return "\n.RS\n.sp\n.nf\n$b\n.fi\n.RE\n" }
    proc href {t h}	{ return "$t ($h) " }
    proc mailaddress {a} { return "<$a>" }
    proc break {}	{ return "\n.br\n" }

    set docstring ".\\\"\n.\\\" Generated by cceGenerateDoc\n.\\\"\n[expr \"$doc\"]\n"

    # Strip redundant whitespace

    regsub -all "\n\[\n \]*" $docstring "\n" docout
    regsub -all -- "-" $docout "\\-" docout

    return $docout
}

# Shift the command-line arguments left by one
proc shift {} {
    global argv
    set argv [lrange $argv 1 end]
}

# Get the date in a reasonable format

if {[info commands clock] == "clock" } {

    # Have the clock command--use it to get the time

    set datestring [clock format [clock seconds] -format "%e %B %Y"]
    
} else {

    # Don't have the clock command--use the date program

    set date [exec date "+%d %m %y"]
    set day [lindex $date 0]
    set month [lindex $date 1]
    regsub "^0" $month "" month
    regsub "^0" $day "" day
    set month [lindex \
	       "- January February March April May June July August September October November December" \
		   $month]
    set datestring "$day $month 19[lindex $date 2]"

}

while {[llength $argv] > 0} {
    switch -- [lindex $argv 0] {
	--html {
	    puts [generate_html_documentation $documentation]	
	}
	--man {
	    puts [generate_troff_documentation $documentation]
	}
    }
    shift
}

# Local Variables:
# mode: tcl
# End:
